#ifndef _ADC_H_
#define _ADC_H_

#ifdef __cplusplus
extern "C"	{
#endif

enum adc_channel	{
	adc_ch_350V,		// 0
	adc_ch_12V,			// 1
	adc_ch_Iout,		// 2
	adc_ch_Iin,			// 3
	_adc_ch_last_,		// 4
	
	_adc_ch_next_power_of_2= 4,
};

#define ADC_CHANNELS _adc_ch_last_
#define ADC_PIN_MASK ((1 << _adc_ch_last_)-1)

enum adc_ref_source	{
	adc_ref_external=	0,
	adc_ref_avcc=		(1<<REFS0),
	adc_ref_internal=	((1<<REFS1) | (1<<REFS0)),
};

#define ADC_REF_SOURCE adc_ref_internal

#define ADC_SAMPLE_MICROS(mean) (100 * ADC_CHANNELS * mean)	/* 10kHz sind 100s */

static inline uint16_t ADC_SAMPLE_NUM(uint8_t mean, uint16_t milliSecs)
{
	return 1000UL * milliSecs / ADC_SAMPLE_MICROS(mean);
}

#define trigger_adc() ADCSRA |= (1<<ADSC)


#ifdef __cplusplus
}

#include "MeasResult.h"

extern "C" void ADC_vect(void) __attribute__ ((signal));

class TAdcScaler	{
// sizeof von Power of 2 anstreben!
private:
	volatile uint8_t MeanCount;		// 1
	volatile uint16_t MeanSum;		// 3
	
	uint8_t MeanMax;				// 4
	uint16_t Dividend;				// 6
	uint16_t Divisor;				// 8
	
	friend void ADC_vect(void);		// ISR handler ist friend!
	
public:
	void SetScaler(uint8_t mean, uint16_t _max);
	TMeasResult GetMean(bool round);
	
	static void Init();
	static void Start();
};	
extern TAdcScaler AdcValues[ADC_CHANNELS];
#endif	/* #ifdef __cplusplus */

#endif /* _ADC_H_ */