#ifndef _MEASVALUE_H_
#define _MEASVALUE_H_

#define WITH_HYSTERESIS
class TMeasValue	{
	struct TInfo {
		uint8_t		Digits:	2;
		uint8_t		Divide:	2;
		uint8_t		Flags:	4;
	};
	
	enum eFlag	{
		flMeas= 1,
		flCalc= 2,
		flAll= 15,
	};		
	
#ifdef WITH_HYSTERESIS
	uint8_t		HystSpan;
	uint16_t	HystLow, HystUp;
#endif
protected:
//	static		char LcdBuff[16+1];
	TInfo		Info;
	uint16_t	Value;
	PGM_P		DescrTx;
	PGM_P		UnitTx;
	
	uint8_t Format(uint16_t value, char* buff) const;
	
public:
	TMeasValue(uint8_t digits, uint8_t divide, uint8_t hysteresis, 
		PGM_P descrTx, PGM_P unitTx);
	virtual bool SetValue(uint16_t value);
	bool Print(uint8_t x, uint8_t y);
	bool HasCalcValue() const	{ return (Info.Flags & flCalc)!=0; }
	uint16_t GetCalcValue();
	void ClearValue()			{ Info.Flags= 0; }
};	

class TMonitorValue : public TMeasValue	{
public:
	enum Directions	{ Minus= -1, Plus= 1, };
		
private:
	uint16_t	Limit, Limit2;
	uint16_t	MaxCount, MaxCount2;
	uint16_t	Count,	Count2;
	Directions	Dir;
	PGM_P		ErrorTx;
	
	inline uint8_t GetCompareResult() const
	{
		if ((MaxCount2 != 0) && (Count2 >= MaxCount2)) return 2;
		if (Count >= MaxCount) return 1;
		return 0;
	}

public:	
	TMonitorValue(uint8_t digits, uint8_t divide, uint8_t hysteresis, 
		PGM_P descrTx, PGM_P unitTx, PGM_P errorTx);
	inline void Disable()			{ MaxCount= 0; }
	inline void Reset()				{ Count= Count2= 0; }	
	void SetMonitor(uint16_t limitValue, Directions direction, uint16_t timeTicks, 
		uint8_t strictPercent= 0, uint16_t strictTicks= 0);
	virtual bool SetValue(uint16_t value);
	void PrintLastError1(uint8_t x, uint8_t y) const;
	void PrintLastError2(uint8_t x, uint8_t y) const;
};	
#endif /* _MEASVALUE_H_ */