#include <avr/io.h>
#include <avr/pgmspace.h>

#include <string.h>
#include <stdlib.h>

extern "C" {
#include "lcd.h"
};

#include "Menue.h"
#include "Utils.h"

extern char LcdBuff[];

void TMenuContext::StartEdit(const TMenuItem& item, uint8_t x, uint8_t y)
{
	pDef = &item;
	uint8_t* addr = pgm_read_ptr(pDef->EEAddr);
	Value = *addr;

	// wichtige Werte aus der (ROM-) Definition bernehmen
	Flags = pgm_read_byte(&(pDef->Flags));
	InvVal = pgm_read_byte(&(pDef->InvVal));

	if (!HoldsInvalid())
	{
		uint8_t minVal = pgm_read_byte(&(pDef->MinVal));
		if (Value < minVal) Value = minVal;
		uint8_t maxVal = pgm_read_byte(&(pDef->MaxVal));
		if (Value > maxVal) Value = maxVal;
	}
	lcd_clear();
	lcd_setcursor(x, y);
	PGM_P descr = pgm_read_ptr(pDef->DescrTx);
	x += lcd_string_P(descr);
	while (x++ < 16) lcd_data(' ');
}

bool TMenuContext::EndEdit()
{
	uint8_t* addr = pgm_read_ptr(pDef->EEAddr);
	if (*addr == Value) return false;	// no change
	*addr = Value;
	return true;						// changed
}

void TMenuContext::Print(uint8_t x, uint8_t y)
{
	FP_MenuFormat fmt = pgm_read_ptr(pDef->Format);
	fmt(*this, LcdBuff);
	lcd_setcursor(x, y);
	x += lcd_string(LcdBuff);
	if (!HoldsInvalid())
	{
		PGM_P unit = pgm_read_ptr(pDef->UnitTx);
		if (unit) x += lcd_string_P(unit);
	}
	while (x++ < 16) lcd_data(' ');
}

void TMenuContext::DoEdit(Directions dir)
{
	uint8_t minVal = pgm_read_byte(&(pDef->MinVal));
	uint8_t maxVal = pgm_read_byte(&(pDef->MaxVal));
	uint8_t step = pgm_read_byte(&(pDef->Step));
	int8_t x = step;
	x *= (int8_t)dir;
	int16_t val = Value;

	if (Flags & mnfInvDown && val == InvVal)
	{
		// invalid unten
		if (x > 0) val = minVal;		// wieder auf valid setzen
		x = 0;						// decrement blockieren
	}
	else if (Flags & mnfInvUp && val == InvVal)
	{
		// invalid oben
		if (x < 0) val = maxVal;		// wieder auf valid setzen
		x = 0;							// increment blockieren
	}
	else
	{
		val += x;
		if (val < minVal)
		{
			if (Flags & mnfInvDown) val = InvVal;
			else if (Flags & mnfWrap) val = maxVal;
			else val = minVal;
		}
		else if (val > maxVal)
		{
			if (Flags & mnfInvUp) val = InvVal;
			else if (Flags & mnfWrap) val = minVal;
			else val = maxVal;
		}
		else
			val -= val % step;		// Einrasten auf Step
	}
	Value = (uint8_t)val;
}