#ifndef _STATUS_H_
#define _STATUS_H_

enum eMachineStates	{
	mstReady,
	mstStarting,
	mstReg,
	mstStopping,
	
	mstError= 128,
};


class TStateMachine {
private:
	typedef void (*FP_StateAction)(TStateMachine& s, uint8_t what);
	
	eMachineStates	State;
	bool			AutoStart;
	
	uint16_t		Time;
	uint16_t		MaxTime;
	int16_t			CompVolt;
	int16_t			StartVolt;
	uint8_t			What;
	FP_StateAction	Action;
	
	void SetState(eMachineStates new_state);
	void StopImmediate();
	void StopSoft();
	static void AcOn(TStateMachine& s, uint8_t what);
	static void AcOff(TStateMachine& s, uint8_t what);
	static void VoltageUp(TStateMachine& s, uint8_t what);
	
public:
	TStateMachine();
	void SetTimer(uint16_t ticks, uint8_t what, int16_t comp_voltage, FP_StateAction action);
	eMachineStates GetState()	const		{ return State; }
	void Start();
	void Stop();
	eMachineStates DoTimer(int16_t voltage);
	
	static uint8_t IsError(uint8_t state)
	{
		return state >= mstError;
	}
	
	void SetError(eMachineStates error_state)
	{
		if (IsError(error_state)) SetState(error_state);
	}
};
extern TStateMachine MachineState;

#endif	/* #ifndef _STATUS_H_ */