#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include "ac.h"
#include "adc.h"
#include "keys.h"

#if F_CPU!=20000000UL
 #error Werte sind auf 20MHz ausgelegt
#endif


#define SINE_STEPS 200
#define SINE_RELOAD (1000 - 1)

volatile uint16_t timer_100;				// 100 Hz increment

static const uint16_t sine_tab[SINE_STEPS] PROGMEM = {
#include "sinus200.cin"
};

// Die PWM wird invertiert betrieben, um einen Takt komplett inaktiv zu haben:
// Setting the OCR1x equal to TOP will result in a constant high or low output 
// (depending on the polarity of the output set by the COM1x1:0 bits.)
// If the OCR1x is set equal to BOTTOM (0x0000) the out-put will be a narrow spike for each TOP+1 timer clock cycle
// -> Setzen von SINE_RELOAD als PWM macht einen komplett inaktiven Step
// -> Setzen von 0 als Reload gibt einen kurzen Einbruch bei PWM max

// Aufruf mit 20kHz
ISR(TIMER1_CAPT_vect)
{
	// Zeit gemessen: 2s (ohne ISR Frame)

	static uint8_t step = 0;
	uint8_t s = step;						// step = Ringzhler 0 .. SINE_STEPS-1

	if ((s & 1) == 0) trigger_adc();			// mu mit Frequenz 10kHz getriggert werden!

	if (s == SINE_STEPS - 1) s = 0;
	else if (s++ == 1)						// Oszilloskop: 1 ist besser als 0 !!!
	{
		// der inaktive Step wird gerade ausgegeben
		// -> Polarittswechsel
		AC_CTRL_PIN |= (1 << AC_POLARTIY_PIN); // Atmega: Writing a logic one to PINxn toggles the value of PORTxn
	}
	step = s;
	OCR1A = pgm_read_word(&sine_tab[s]);

	if (s == 0)
	{
		// Hier haben wir auch 100 Hz die wir fr das interne Timing nutzen knnen
		// Besser erst ausfhren, nachdem der Timer neu geladen wurde (zeitkritisch)
		timer_100++;

		// every 10ms
		key_isr();
	}
}

static void init_timer1()
{
	// Timer1 macht die Sinuserzeugung mit einer PWM Frequenz von 20kHz
	// die PWM Reload-Werte wirken invertiert, d.h. der grte Wert wirkt als krzeste tOn
	// -> der PWM-Ausgang wird invertiert betrieben (low-aktiv)
	// PWM an OC1A, 
	// Fast PWM TOP=ICR1 = WGM 14
	TCCR1A = (1 << COM1A1) | (1 << WGM11);
	TCCR1B = (1 << WGM12) | (1 << WGM13);
	ICR1 = SINE_RELOAD;
	OCR1A = SINE_RELOAD;
	AC_CTRL_DDR |= (1 << AC_PWM_PIN);			// OC1A PIN als output -> PWM
}


void init_ac()
{
	// Steuer-Ausgnge als Output, Pegel Low
	AC_CTRL_PORT &= ~((1 << AC_POLARTIY_PIN) | (1 << AC_ENABLE_PIN)); // disable_ac();
	AC_CTRL_DDR |= (1 << AC_POLARTIY_PIN) | (1 << AC_ENABLE_PIN);
	init_timer1();
}

void start_ac()
{
	TIMSK1 |= (1 << ICIE1);					// Timer/Counter1, input capture interrupt enable		
	TCCR1B |= (1 << CS10);					// Timer starten mit Teiler 1 
}
