#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/crc16.h>

#include "eedata.h"

EEData eeData;

EEData EEMEM eeStore = {
	-1,			// max_inp_current
	-1,			// max_outp_current
	-1,			// min_inp_voltage
	-1,			// min_ece
	0,			// auto_on
};

static uint16_t get_crc(const EEData* pData)
{
	uint16_t crc = 0x2605;
	uint8_t* pBuff = (uint8_t*)pData;
	uint8_t i = sizeof(*pData) - sizeof(pData->crc);
	do {
		crc = _crc16_update(crc, *pBuff++);
	} while (--i);
	return crc;
}

void write_ee(EEData* pData)
{
	pData->crc = get_crc(pData);
	eeprom_write_block(pData, &eeStore, sizeof(*pData));
}

uint8_t init_ee(EEData* pData)
{
	eeprom_read_block(pData, &eeStore, sizeof(*pData));
	if (get_crc(pData) != pData->crc)
	{
		pData->max_inp_current = ~0;
		pData->max_outp_current = ~0;
		pData->min_inp_voltage = ~0;
		pData->min_ece = ~0;
		pData->auto_on = 0;
		write_ee(pData);
		return 0;
	}
	return 1;
}


