#include <avr/io.h>
#include <util/atomic.h>

#include "keys.h"

volatile uint8_t key_state;	
volatile uint8_t key_press;	
volatile uint8_t key_rpt;	
volatile uint8_t key_repeat_mask;

///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press( uint8_t key_mask )
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{													// read and clear atomic !
		key_mask &= key_press;                          // read key(s)
		key_press ^= key_mask;                          // clear key(s)
	}
	return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//
uint8_t get_key_rpt( uint8_t key_mask )
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{											// read and clear atomic !
		key_mask &= key_rpt;					// read key(s)
		key_rpt ^= key_mask;					// clear key(s)
	}	
	return key_mask;
}
 
#if 0
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_short( uint8_t key_mask )
{
	uint8_t res;
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{											// read key state and key press atomic !
		res= get_key_press( ~key_state & key_mask );
	}
	return res;	
}
 
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_long( uint8_t key_mask )
{
	return get_key_press( get_key_rpt( key_mask ));
}
#endif
