#ifndef KEYS_H_
#define KEYS_H_

#define KEY_PORT PORTB
#define KEY_PIN PINB
#define KEY_DDR DDRB

#define KEY_ON          4		/* PB4 */
#define KEY_OFF         5		/* PB5 */
#define KEY_MENU        3		/* PB3 */
#define ALL_KEYS        (1<<KEY_ON | 1<<KEY_OFF | 1<<KEY_MENU)
 
#define REPEAT_START    50					/* after 500ms */
#define REPEAT_NEXT     20					/* every 200ms */

static inline void init_keys()
{
	KEY_PORT|= ALL_KEYS;		// Pull-ups on
	KEY_DDR&= ~ALL_KEYS;		// Input
}

extern volatile uint8_t key_repeat_mask;
extern volatile uint8_t key_state;									// debounced and inverted key state:
																	// bit = 1: key pressed
extern volatile uint8_t key_press;									// key press detect
extern volatile uint8_t key_rpt;									// key long press and repeat


static inline void set_key_repeat(uint8_t keys)
{
	key_repeat_mask= keys;
}

static inline void key_isr()
{
	static uint8_t ct0, ct1, rpt;

	uint8_t ks= key_state;			// buffer volatile
	
	uint8_t i = ks ^ ~KEY_PIN;				// key changed ?
	ct0 = ~( ct0 & i );						// reset or count ct0
	ct1 = ct0 ^ (ct1 & i);					// reset or count ct1
	i &= ct0 & ct1;							// count until roll over ?
	
	ks ^= i;								// then toggle debounced state
	key_press |= ks & i;					// 0->1: key press detect
	
	#define REPEAT_MASK rm
	uint8_t rm= key_repeat_mask;	// buffer volatile
	
	if( (ks & REPEAT_MASK) == 0 )			// check repeat function
		rpt = REPEAT_START;					// start delay
	if( --rpt == 0 )
	{
		rpt = REPEAT_NEXT;					// repeat delay
		key_rpt |= ks & REPEAT_MASK;
	}
	key_state= ks;
}

extern uint8_t get_key_press( uint8_t key_mask );
extern uint8_t get_key_rpt( uint8_t key_mask );
extern uint8_t get_key_short( uint8_t key_mask );
extern uint8_t get_key_long( uint8_t key_mask );

#endif /* KEYS_H_ */